#ifndef __BP35A1_CLIENT_H__
#define __BP35A1_CLIENT_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <termios.h>
#include <unistd.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <glib-object.h>
#include <glib.h>
#include "bp35a1-message.h"
#include "bp35a1-raw-message.h"

G_BEGIN_DECLS

#define BP35A1_TYPE_CLIENT            (bp35a1_client_get_type ())
#define BP35A1_CLIENT(o)              (G_TYPE_CHECK_INSTANCE_CAST  ((o), BP35A1_TYPE_CLIENT, Bp35a1Client))
#define BP35A1_IS_CLIENT(o)           (G_TYPE_CHECK_INSTANCE_TYPE  ((o), BP35A1_TYPE_CLIENT))
#define BP35A1_CLIENT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), BP35A1_TYPE_CLIENT, Bp35a1ClientClass))
#define BP35A1_IS_CLIENT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BP35A1_TYPE_CLIENT))
#define BP35A1_CLIENT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), BP35A1_TYPE_CLIENT, Bp35a1ClientClass))

#define BP35A1_CLIENT_DEVICE "/dev/ttymxc0"
#define BP35A1_CLIENT_READ_BUF_SIZE       (256)
#define BP35A1_CLIENT_READ_TIMEOUT_SEC    (0)
#define BP35A1_CLIENT_READ_TIMEOUT_USEC   (500)
#define BP35A1_CLIENT_ECHONET_LITE_HANDLE (1)

#define BP35A1_CLIENT_REG_CH "35"
#define BP35A1_CLIENT_REG_PAN_ID "1234"
#define BP35A1_CLIENT_REG_HAN_ID "DDLeDisp"
#define BP35A1_CLIENT_REG_BEACON_REQ "1"

typedef struct {
  gchar *name;
  gchar *value;
} Bp35a1Register;

typedef struct {
  GObject parent_instance;

  int fd;
  gchar *device_name;
  guint read_buf_size;;
  struct termios old_tio;
  struct timeval read_timeout;
  GList *register_table;
  GList *whitelist;

  gchar encrypt_option[2];
} Bp35a1Client;

typedef struct {
  GObjectClass parent_class;
} Bp35a1ClientClass;

GType            bp35a1_client_get_type (void) G_GNUC_CONST;
Bp35a1Client *   bp35a1_client_new (void);
gint             bp35a1_client_start (Bp35a1Client *client);
gint             bp35a1_client_stop (Bp35a1Client *client);
gint             bp35a1_client_post_message (Bp35a1Client * client, Bp35a1Message * message);
Bp35a1Message ** bp35a1_client_receive_messages (Bp35a1Client * client);
GByteArray *     bp35a1_client_get_message_data (Bp35a1Client * client, Bp35a1Message * message);
GByteArray *     bp35a1_client_get_message_data_permittable (Bp35a1Client * client, Bp35a1Message * message, gboolean * permit);
void             bp35a1_client_debug_dump_registers (Bp35a1Client * client);
Bp35a1Message *  bp35a1_client_new_response_message (Bp35a1Client * client, Bp35a1Message * message, GByteArray * data);
gint             bp35a1_client_whitelist_add(Bp35a1Client * client, const gchar * item);
gint             bp35a1_client_whitelist_delete(Bp35a1Client * client, const gchar * item);
gboolean         bp35a1_client_whitelist_is_exist(Bp35a1Client * client, const gchar * item);
gint             bp35a1_client_whitelist_get_index(Bp35a1Client * client, const gchar * item);

gchar * bp35a1_client_get_device_name (Bp35a1Client *client);
void    bp35a1_client_set_device_name (Bp35a1Client *client, const gchar *string);
guint   bp35a1_client_get_read_buf_size (Bp35a1Client *client);
void    bp35a1_client_set_read_buf_size (Bp35a1Client *client, guint size);


G_END_DECLS

#endif /* __BP35A1_CLIENT_H__ */
