#include "bp35a1-raw-message.h"

Bp35a1RawMessage *bp35a1_raw_message_new (gpointer data, gsize size)
{
  Bp35a1RawMessage *raw;
  raw = g_slice_new (Bp35a1RawMessage);
  raw->size = size;

  raw->data = g_malloc (raw->size);
  memcpy (raw->data, data, raw->size);

  return raw;
}

void bp35a1_raw_message_free (Bp35a1RawMessage * raw)
{
  if (raw != NULL) {
    if (raw->data != NULL) {
      g_free (raw->data);
      raw->data = NULL;
    }
    raw->size = 0;
    g_slice_free (Bp35a1RawMessage, raw);
  }
}

void bp35a1_raw_message_freev (Bp35a1RawMessage ** raw_array)
{
  gint i;

  if (raw_array) {
    for (i = 0; raw_array[i] != NULL; i++)
      bp35a1_raw_message_free (raw_array[i]);

    g_free (raw_array);
  }
}

gchar *bp35a1_raw_message_to_string (Bp35a1RawMessage * raw)
{
  g_return_val_if_fail (raw != NULL, NULL);

  gchar *str;
  str = g_new (gchar, raw->size + 1);
  memcpy (str, raw->data, raw->size);
  str[raw->size] = 0x00;

  return str;
}

Bp35a1RawMessage **bp35a1_raw_message_split (Bp35a1RawMessage * raw,
    const gchar *delimiter, gint max_tokens)
{
  GSList *raw_list = NULL, *rlist;
  Bp35a1RawMessage **raw_array;
  gpointer remainder, s;
  guint n = 0;
  gint delimiter_len, len;

  if (max_tokens < 1)
    max_tokens = G_MAXINT;

  remainder = raw->data;
  s = (gpointer) strstr (remainder, delimiter);

  if (s) {
    delimiter_len = strlen (delimiter);

    while (--max_tokens && s) {
      len = s - remainder;
      raw_list = g_slist_prepend (raw_list,
          bp35a1_raw_message_new (remainder, len));
      n++;
      remainder = s + delimiter_len;
      s = (gpointer) strstr (remainder, delimiter);
    }
  }

  len = raw->size - (remainder - raw->data);
  if (len > 0) {
    n++;
    raw_list = g_slist_prepend (raw_list,
        bp35a1_raw_message_new (remainder, len));
  }

  raw_array = g_new (Bp35a1RawMessage *, n + 1);

  raw_array[n--] = NULL;
  for (rlist = raw_list; rlist; rlist = rlist->next)
    raw_array[n--] = rlist->data;

  g_slist_free (raw_list);

  return raw_array;

}
