#ifndef _EDISPZ_CONTROLLER_H_
#define _EDISPZ_CONTROLLER_H_

#include <glib.h>
#include <glib/gstdio.h>
#include "bp35a1-client.h"
#include "bp35a1-message.h"

#define EDISPZ_HEADER_SIZE (15)

typedef struct {
  struct _first
  {
    guchar fixed[2];
    guchar tid[2];
  } first;
  struct _second
  {
    guchar command;
    guchar serial[8];
    guchar resv;
    guchar payload_len;
  } second;
} EdispzHeader;

typedef struct {
  gchar* mac;
  GByteArray *data;
} EdispzPacket ;

typedef struct {
  gchar *device;
  GAsyncQueue *dest_queue;

  Bp35a1Client *bp35a1_client;

  guchar tid_up, tid_low;
  guint interval_time;
  guint offset_time;
  guint human_sensor_wait_time;
  gfloat temperature_upper;
  gfloat temperature_lower;
  gfloat humidity_upper;
  gfloat humidity_lower;
  gushort heartbeat_time;
  guchar door_detect_mode;
  guchar pana_psk[16];
  guchar operation_mode;

  GMutex lock;
  GThread *loop;
  gboolean is_running;
} EdispzController;

typedef enum {
  EDISPZ_PACKET_ACK_NACK = 0x3,
  EDISPZ_PACKET_TIMEKEEPING = 0x7,
  EDISPZ_PACKET_SPECIAL = 0x9,
  EDISPZ_PACKET_TEST = 0xB,
  EDISPZ_PACKET_NORMAL = 0xC,
  EDISPZ_PACKET_BROKEN = 0xFE,
  EDISPZ_PACKET_UNKNOWN = 0xFF,
} EdispzPacketType;

#define EDISPZ_PACKET_ACK_NACK_LENGTH (1)
#define EDISPZ_PACKET_NORMAL_LENGTH (33)
#define EDISPZ_PACKET_TEST_LENGTH (42)
#define EDISPZ_PACKET_SPECIAL_LENGTH (42)
#define EDISPZ_PACKET_TIMEKEEPING_LENGTH (38)

typedef enum {
  EDISPZ_PACKET_DATA_UNKNOWN,
  EDISPZ_PACKET_DATA_ACK_NACK,
  EDISPZ_PACKET_DATA_BATTERY,
  EDISPZ_PACKET_DATA_DOOR_IS_OPEN,
  EDISPZ_PACKET_DATA_DOOR_OPEN_COUNT,
  EDISPZ_PACKET_DATA_DOOR_TIME,
  EDISPZ_PACKET_DATA_TEMPERATURE,
  EDISPZ_PACKET_DATA_HUMIDITY,
  EDISPZ_PACKET_DATA_HUMAN_IS_REACT,
  EDISPZ_PACKET_DATA_HUMAN_REACT_COUNT,
  EDISPZ_PACKET_DATA_FIRMWARE_VERSION,
  EDISPZ_PACKET_DATA_TIMEKEEPING_FLAG,
  EDISPZ_PACKET_DATA_FIRMWARE_SEQUENCE_NUMBER,
  EDISPZ_PACKET_DATA_CO2,
  EDISPZ_PACKET_DATA_ANALOG_INPUT,
  EDISPZ_PACKET_DATA_SCALE,
  EDISPZ_PACKET_DATA_PULSE
} EdispzPacketDataType;

EdispzController *edispz_controller_new   (GAsyncQueue *dest_queue);
gint              edispz_controller_start (EdispzController *controller);
gint              edispz_controller_stop  (EdispzController *controller);
void              edispz_controller_free  (EdispzController *controller);
EdispzPacketType  edispz_packet_get_type  (GByteArray *packet);
gchar            *edispz_packet_get_data  (GByteArray *packet, EdispzPacketDataType data_type);
void              delete_edispz_packet    (EdispzPacket *packet);

#endif
